/*

    Zelda Return of the Hylian

    Copyright (C) 2005-2008  Vincent Jouillat

    Please send bugreports with examples or suggestions to www.zeldaroth.fr

*/

#include <sstream>
#include <fstream>
#include <iostream>

#include <SDL/SDL.h>

#include "Audio.h"

Audio::Audio() : volume(0), musiqueId(0), specialId(0) {
    SOUND = FSOUND_Init(44100, 32, 0);
    music = NULL;
    
    if (SOUND) {
        previous_volume = -1;
        previous_volson = FSOUND_GetSFXMasterVolume();
        loadSounds();
    }
}

Audio::~Audio() {
    if (SOUND) {
        freeSounds();
        FMUSIC_StopSong(music);
        FMUSIC_SetMasterVolume(music, previous_volume);
        FMUSIC_FreeSong(music);
        FSOUND_SetSFXMasterVolume(previous_volson);
        FSOUND_Close();
    }
}

void Audio::setVolume(int vol) {volume=vol*4;
    if (previous_volume == -1) previous_volume = FMUSIC_GetMasterVolume(music);
    FMUSIC_SetMasterVolume(music, volume);}
void Audio::setVolson(int volson) {FSOUND_SetSFXMasterVolume(volson*4);
   // for (int i = 0; i < 40; i++) Mix_VolumeChunk(sons[i], volson);
}

void Audio::loadSounds() {
    sons = new FSOUND_SAMPLE*[40];
    
    sons[0] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/text.ogg",0,0,0); // lettres
    sons[1] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/menu1.ogg",0,0,0); // menu 1
    sons[2] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/menu2.ogg",0,0,0); // menu 2
    sons[3] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/menu3.ogg",0,0,0); // menu 3
    sons[4] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/menu4.ogg",0,0,0); // menu 4
    sons[5] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/timewarp.ogg",0,0,0); // time retour
    sons[6] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/tombe.ogg",0,0,0); // tombe (ennemi)
    sons[7] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/hitenemy.ogg",0,0,0); //shot 1
    sons[8] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/killenemy.ogg",0,0,0); // shot 2
    sons[9] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/surprise.ogg",0,0,0); // surprise
    sons[10] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/monte.ogg",0,0,0); // monte
    sons[11] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/descend.ogg",0,0,0); // descend
    sons[12] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/chute.ogg",0,0,0); // chute
    sons[13] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/item.ogg",0,0,0); // item
    sons[14] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/rupee.ogg",0,0,0); // rubis
    sons[15] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/heart.ogg",0,0,0); // coeur
    sons[16] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/bomb.ogg",0,0,0); // bombe
    sons[17] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/textnext.ogg",0,0,0); // suite texte
    sons[18] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/textend.ogg",0,0,0); // fin texte
    sons[19] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/happy.ogg",0,0,0); // trouve objet
    sons[20] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/door.ogg",0,0,0); // ouvre porte
    sons[21] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/pics.ogg",0,0,0); // pics contre mur
    sons[22] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/sword.ogg",0,0,0); // pe
    sons[23] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/SwordCharging.ogg",0,0,0); // charge
    sons[24] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/Sword360.ogg",0,0,0); // spin
    sons[25] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/shoot.ogg",0,0,0); // flche
    sons[26] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/hookshot.ogg",0,0,0); // grappin
    sons[27] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/stamp.ogg",0,0,0); // pose bombe
    sons[28] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/magic.ogg",0,0,0); // magie
    sons[29] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/burn.ogg",0,0,0); // brle
    sons[30] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/hammer.ogg",0,0,0); // marteau
    sons[31] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/plouf.ogg",0,0,0); // plouf
    sons[32] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/danger.ogg",0,0,0); // danger
    sons[33] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/hurt.ogg",0,0,0); // link se blesse
    sons[34] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/porte.ogg",0,0,0); // porte objet
    sons[35] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/lance.ogg",0,0,0); // lance objet
    sons[36] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/casse.ogg",0,0,0); // casse objet
    sons[37] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/charge.ogg",0,0,0); // charge magie
    sons[38] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/buisson.ogg",0,0,0); // coupe buisson
    sons[39] = FSOUND_Sample_Load(FSOUND_FREE, "data/sound/pousse.ogg",0,0,0); // pousse caisse
    
}

void Audio::freeSounds() {
    if (SOUND) {
        for (int i = 0; i < 40; i++) FSOUND_Sample_Free(sons[i]);
        delete[] sons;
    }
}

void Audio::playSound(int id, int chl) {
    if (SOUND) //Mix_PlayChannel(chl,sons[id],0);
        FSOUND_PlaySound(chl, sons[id]);
}

void Audio::playMusic(int id) {
    if (SOUND) {
        if (musiqueId != id) {
            musiqueId = id;            
            if (specialId == 0) {
                FMUSIC_StopSong(music);
                FMUSIC_FreeSong(music);
                music = choixMusique(id);
                if (previous_volume == -1) previous_volume = FMUSIC_GetMasterVolume(music);
                FMUSIC_SetMasterVolume(music, volume);
                FMUSIC_SetLooping(music, 1);
                FMUSIC_PlaySong(music);
                specialId = 0;
            }
        }
    }
}

void Audio::stopMusic() {
    if (SOUND) FMUSIC_StopSong(music);
}

void Audio::replayMusic() {
    if (SOUND) FMUSIC_PlaySong(music);
}

FMUSIC_MODULE* Audio::choixMusique(int id) {
    switch (id) {
        case 1 : return FMUSIC_LoadSong("data/music/Foret.mid");
        case 2 : return FMUSIC_LoadSong("data/music/Plaine.mid");
        case 3 : return FMUSIC_LoadSong("data/music/Lac.mid");
        case 4 : return FMUSIC_LoadSong("data/music/Mont.mid");
        case 5 : return FMUSIC_LoadSong("data/music/Desert.mid");
        case 6 : return FMUSIC_LoadSong("data/music/Ombre.mid");
        case 7 : return FMUSIC_LoadSong("data/music/Feu.mid");
        case 8 : return FMUSIC_LoadSong("data/music/Cocorico.mid");
        case 9 : return FMUSIC_LoadSong("data/music/Chateau.mid");
        case 10 : case 11 : return FMUSIC_LoadSong("data/music/Secret.mid");
        case 12 : case 13 : case 14 : return FMUSIC_LoadSong("data/music/Donjon1.mid");
        case 15 : return FMUSIC_LoadSong("data/music/DDesert.mid");
        case 16 : case 17 : case 18 : return FMUSIC_LoadSong("data/music/Donjon2.mid");
        case 19 : return FMUSIC_LoadSong("data/music/DSecret.mid");
        case 20 : return FMUSIC_LoadSong("data/music/DFinal.mid");
        case 21 : return FMUSIC_LoadSong("data/music/Home.mid");
        case 22 : case 23 : return FMUSIC_LoadSong("data/music/Cave.mid");
        case 24 : case 25 : case 26 : case 27 : return FMUSIC_LoadSong("data/music/Fee.mid"); break;
//case 28 : case 32 : case 35 : case 36 : case 38 : case 41 : case 42 : case 43 : case 44 :
//    return FMUSIC_LoadSong("data/music/Maison.mid"); break;
        case 29 : case 37 : case 40 : return FMUSIC_LoadSong("data/music/Potion.mid"); break;
        case 30 : return FMUSIC_LoadSong("data/music/Bar.mid"); break;
        case 31 : return FMUSIC_LoadSong("data/music/Magasin.mid"); break;
        case 33 : case 34 : case 39 : return FMUSIC_LoadSong("data/music/Jeu.mid"); break;
        case 45 : return FMUSIC_LoadSong("data/music/Titre.mid"); break;
        case 46 : return FMUSIC_LoadSong("data/music/Selection.mid"); break;
        case 47 : return FMUSIC_LoadSong("data/music/Debut.mid"); break;
        default : return FMUSIC_LoadSong("data/music/Maison.mid");
    }
}

void Audio::playSpecial(int id) {
    if (SOUND) {
        if (specialId != id) {
            FMUSIC_StopSong(music);
            FMUSIC_FreeSong(music);
            music = choixSpecial(id);
            if (previous_volume == -1) previous_volume = FMUSIC_GetMasterVolume(music);
            FMUSIC_SetMasterVolume(music, volume);
            FMUSIC_SetLooping(music, 1);
            FMUSIC_PlaySong(music);
            specialId=id;
        }    
    }
}

void Audio::stopSpecial() {
    if (!specialId) return;
    int tmp = musiqueId;
    musiqueId = 0;
    specialId = 0;
    playMusic(tmp);
}

FMUSIC_MODULE* Audio::choixSpecial(int id) {
    switch (id) {
        case 1 : return FMUSIC_LoadSong("data/music/Boss.mid");
        case 2 : return FMUSIC_LoadSong("data/music/Mort.mid");
        case 3 : return FMUSIC_LoadSong("data/music/Epee.mid");
        case 4 : return FMUSIC_LoadSong("data/music/BossF.mid");
        case 5 : return FMUSIC_LoadSong("data/music/Fin.mid");
        default : return FMUSIC_LoadSong("data/music/Boss.mid");
    }
}

